// uli_test.c
//
// Test ULI handler for BC535PCI 1-PPS interrupt
//


#include <stdio.h>
#include <time.h>
#include <unistd.h>

#include <sys/mman.h>
#include <sys/time.h>

#include "stfplib.h"


#ifndef TRUE
#define TRUE 1
#endif
#ifndef FALSE
#define FALSE 0
#endif


const char*  stfp_base_pathname = "/hw/module/001c02/Ibrick/xtalk/14/pci/1/ppb_16/0";
static const int   seconds_to_sleep = 20;

static long  interrupt_count = 0;


static void intrfunc(void* arg)
{
  struct stfp_device*  stfp = (struct stfp_device*)arg;

  interrupt_count++;
  stfp_clear_interrupt_status(stfp, 0xffffffff);
}



main(int argc, char *argv[])
{
  struct stfp_device*  stfp;

  printf("bc635PCI ULI Handler Example for IRIX 6.5 \n" );
  printf("----------------------------------------- \n" );


  /* Paging while in the ULI interrupt handler would be fatal so lock entire
   * address space in memory.
   */
  if (mlockall(MCL_CURRENT | MCL_FUTURE))
  {
   perror("mlockall");
  }

  stfp = new_stfp_device(stfp_base_pathname);
  if (stfp == 0)
  {
    printf("Failed to access bc635pci board at %s\n", stfp_base_pathname);
    exit(1);
  }

  /* Make sure interrupts are off.
   */
  stfp_set_interrupt_enable_mask(stfp, 0);
  stfp_clear_interrupt_status(stfp, 0xffffffff);

  if (!stfp_register_interrupt_function(stfp,
                                        intrfunc,
                                        (void*)stfp))
  {
    perror("Error registering interrupt function");
    exit(1);
  }

  printf("Enable interrupts and sleep for about %d seconds...\n", seconds_to_sleep);
  stfp_set_interrupt_enable_mask(stfp, STFP_INTERRUPT_1_PPS_OUTPUT);
  sleep(seconds_to_sleep);
  printf("Sleep ended.\n");

  /* Turn off interrupts.
   */
  stfp_set_interrupt_enable_mask(stfp, 0);
  stfp_clear_interrupt_status(stfp, 0xffffffff);


  delete_stfp_device(&stfp);

  printf("\n Interrupt count = %d\n", interrupt_count);
}
